@@ -1,24 +1,5 @@
-/* Copyright (c) 2020-2022 Yoran Heling
-
-  Permission is hereby granted, free of charge, to any person obtaining
-  a copy of this software and associated documentation files (the
-  "Software"), to deal in the Software without restriction, including
-  without limitation the rights to use, copy, modify, merge, publish,
-  distribute, sublicense, and/or sell copies of the Software, and to
-  permit persons to whom the Software is furnished to do so, subject to
-  the following conditions:
-
-  The above copyright notice and this permission notice shall be included
-  in all copies or substantial portions of the Software.
-
-  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
-  EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
-  MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
-  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
-  CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
-  TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
-  SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
-*/
+/* SPDX-FileCopyrightText: Yorhel <projects@yorhel.nl>
+ * SPDX-License-Identifier: MIT */
 
 #define _GNU_SOURCE
 #include <string.h>
@@ -204,8 +185,13 @@ static void parse_unqarg(struct parse_ctx *ctx) {
                 if(ctx->c == 0)
                     die(ctx->ctx, "Unexpected EOF");
                 break;
-            case '$': /* ${ should not be taken to start a block */
-                if(ctx->buf[ctx->pos+1] == '{')
+            case '$':
+                /* $#{ should not be taken to start a block */
+                if(ctx->buf[ctx->pos+1] == '#' && ctx->buf[ctx->pos+2] == '{') {
+                    parse_take(ctx);
+                    parse_take(ctx);
+                /* nor should ${ */
+                } else if(ctx->buf[ctx->pos+1] == '{')
                     parse_take(ctx);
                 break;
             case 0:
@@ -504,8 +490,10 @@ static void proc_var_del_num(struct proc_ctx *ctx) {
  */
 static char *proc_subst_vars(struct proc_ctx *ctx, const char *str) {
     size_t len = 0, size = strlen(str)+1, varlen;
-    char *var, *cur, *ret = malloc(size);
+    const char *cur;
+    char *var, *ret = malloc(size);
     char quote = *str == '\'' || *str == '"' ? *str : 0;
+    struct cfg_arg *arg;
 
 #define addc(c) do { if(len == size) { size *= 2; ret = realloc(ret, size); } ret[len++] = c; } while(0)
 
@@ -515,19 +503,37 @@ static char *proc_subst_vars(struct proc_ctx *ctx, const char *str) {
             addc(*(str++));
             continue;
         }
+        if(ctx->macro && ctx->macro->array && *str == '$' && *(str+1) == '#') {
+            cur = str + 2 + (*(str+2) == '{');
+            varlen = str_varname(cur);
+            if((*(str+2) != '{' || cur[varlen] == '}') &&
+                    strlen(ctx->macro->array) == varlen &&
+                    memcmp(ctx->macro->array, cur, varlen) == 0) {
+                str += (*(str+2) == '{' ? 2 : 0) + 2 + varlen;
+                varlen = 0;
+                for(arg = ctx->macro_array; arg; arg = arg->next)
+                    varlen++;
+                if(varlen < 10) {
+                    addc(varlen + '0');
+                } else if(varlen < 100) {
+                    addc((varlen / 10) + '0');
+                    addc((varlen % 10) + '0');
+                } else {
+                    fprintf(stderr, "Unable to expand $#{%s}: too many arguments (%d)\n", ctx->macro->array, (int)varlen);
+                    exit(1);
+                }
+                continue;
+            }
+        }
         if(*str == '$') {
-            varlen = str_varname(str + 1 + (*(str+1) == '{'));
-            if((var = proc_var_get(ctx, str + 1 + (*(str+1) == '{'), varlen)) != NULL) {
+            cur = str + 1 + (*(str+1) == '{');
+            varlen = str_varname(cur);
+            if((*(str+1) != '{' || cur[varlen] == '}') && (var = proc_var_get(ctx, cur, varlen)) != NULL) {
+                str += (*(str+1) == '{' ? 2 : 0) + 1 + varlen;
                 cur = var = str_quote(quote, var);
                 while(*cur)
                     addc(*(cur++));
                 free(var);
-                str++;
-                if(*str == '{')
-                    str++;
-                str += varlen;
-                if(*str == '}')
-                    str++;
                 continue;
             }
         }
